@@ -10,39 +10,33 @@ MODEL_LM = "Linea Mini"
 from lmdirect.const import (
     CALCULATED_VALUE,
     CONTINUOUS_OFFSET,
-    DRINKS_K1_OFFSET,
-    DRINKS_K2_OFFSET,
-    DRINKS_K3_OFFSET,
-    DRINKS_K4_OFFSET,
     FLUSHING_OFFSET,
 )
 
+OFFSET = "offset"
+
 DATE_RECEIVED = "date_received"
 POWER = "power"
 POWER_MYSTERY = "power_mystery"
 TEMP_COFFEE = "coffee_temp"
 TEMP_STEAM = "steam_temp"
-TSET_STEAM = "steam_set_temp"
 TSET_COFFEE = "coffee_set_temp"
-DOSE_K1 = "dose_k1"
-DOSE_K2 = "dose_k2"
-DOSE_K3 = "dose_k3"
-DOSE_K4 = "dose_k4"
-DOSE_K5 = "dose_k5"
+TSET_STEAM = "steam_set_temp"
+
+DOSE = "dose"
+
 DOSE_HOT_WATER = "dose_hot_water"
+PREBREW_FLAG = "prebrew_flag"
 ENABLE_PREBREWING = "enable_prebrewing"
-PREBREWING_TON_K1 = "prebrewing_ton_k1"
-PREBREWING_TON_K2 = "prebrewing_ton_k2"
-PREBREWING_TON_K3 = "prebrewing_ton_k3"
-PREBREWING_TON_K4 = "prebrewing_ton_k4"
-PREBREWING_TOFF_K1 = "prebrewing_toff_k1"
-PREBREWING_TOFF_K2 = "prebrewing_toff_k2"
-PREBREWING_TOFF_K3 = "prebrewing_toff_k3"
-PREBREWING_TOFF_K4 = "prebrewing_toff_k4"
-DRINKS_K1 = "drinks_k1"
-DRINKS_K2 = "drinks_k2"
-DRINKS_K3 = "drinks_k3"
-DRINKS_K4 = "drinks_k4"
+ENABLE_PREINFUSION = "enable_preinfusion"
+
+PREBREWING = "prebrewing"
+PREINFUSION = "preinfusion"
+TON = "ton"
+TOFF = "toff"
+
+DRINKS = "drinks"
+
 CONTINUOUS = "continuous"
 TOTAL_COFFEE = "total_coffee"
 HOT_WATER = "hot_water"
@@ -51,7 +45,19 @@ TOTAL_COFFEE_ACTIVATIONS = "total_coffee_activations"
 HOT_WATER_2 = "hot_water_2"
 DRINKS_HOT_WATER = "drinks_hot_water"
 TOTAL_FLUSHING = "total_flushing"
+MACHINE_NAME = "machine_name"
+FRONT_PANEL_DISPLAY = "front_panel_display"
+MYSTERY_VALUES = "mystery_values"
+BREW_GROUP_OFFSET = "brew_group_offset"
+PID_OFFSET = "pid_offset"
+T_UNIT = "t_unit"
+WATER_FILTER_LITERS = "water_filter_liters"
 
+VAL = "val"
+
+MYSTERY_1 = "mystery_1"
+MYSTERY_2 = "mystery_2"
+STEAM_BOILER_ENABLE = "steam_boiler_enable"
 HEATING_STATE = "heating_state"
 STEAM_HEATER_ON = "steam_heater_on"
 COFFEE_HEATER_ON = "coffee_heater_on"
@@ -63,6 +69,7 @@ BOILER_FILL_SOLENOID_ON = "boiler_fill_solenoid_on"
 
 KEY_ACTIVE = "key_active"
 CURRENT_PULSE_COUNT = "current_pulse_count"
+WATER_RESERVOIR_CONTACT = "water_reservoir_contact"
 
 COFFEE_HEATING_ELEMENT_HOURS = "coffee_heating_element_hours"
 STEAM_HEATING_ELEMENT_HOURS = "steam_heating_element_hours"
@@ -76,30 +83,25 @@ MODULE_SER_NUM = "module_ser_num"
 
 UPDATE_AVAILABLE = "update_available"
 
-SUN_ON = "sun_on"
-SUN_OFF = "sun_off"
-MON_ON = "mon_on"
-MON_OFF = "mon_off"
-TUE_ON = "tue_on"
-TUE_OFF = "tue_off"
-WED_ON = "wed_on"
-WED_OFF = "wed_off"
-THU_ON = "thu_on"
-THU_OFF = "thu_off"
-FRI_ON = "fri_on"
-FRI_OFF = "fri_off"
-SAT_ON = "sat_on"
-SAT_OFF = "sat_off"
+ON = "on"
+OFF = "off"
+HOUR = "hour"
+MIN = "min"
+AUTO = "auto"
+TIME = "time"
+
+GLOBAL = "global"
+MON = "mon"
+TUE = "tue"
+WED = "wed"
+THU = "thu"
+FRI = "fri"
+SAT = "sat"
+SUN = "sun"
+
+DAYS = [MON, TUE, WED, THU, FRI, SAT, SUN]
 
 AUTO_BITFIELD = "auto_bitfield"
-GLOBAL_AUTO = "global_auto"
-SUN_AUTO = "sun"
-MON_AUTO = "mon"
-TUE_AUTO = "tue"
-WED_AUTO = "wed"
-THU_AUTO = "thu"
-FRI_AUTO = "fri"
-SAT_AUTO = "sat"
 
 SECOND = "second"
 MINUTE = "minute"
@@ -124,8 +126,12 @@ TYPE_AUTO_ON_OFF = 3
 TYPE_COFFEE_TEMP = 4
 TYPE_STEAM_TEMP = 5
 TYPE_DRINK_STATS = 6
+TYPE_WATER_RESERVOIR_CONTACT = 7
+TYPE_PREINFUSION = 8
+TYPE_START_BACKFLUSH = 9
+TYPE_STEAM_BOILER_ENABLE = 10
 
-DIVIDE_KEYS = ["temp", "prebrewing_to"]
+DIVIDE_KEYS = ["temp", "prebrewing_to", "preinfusion_k", "pid_offset"]
 SERIAL_NUMBERS = [MACHINE_SER_NUM, MODULE_SER_NUM]
 
 # Response Maps
@@ -158,7 +164,10 @@ class Elem:
 # 04 D8: Steam Temp (124.0C)
 # B6: Check byte
 
-TEMP_REPORT_MAP = {Elem(0, 2): TEMP_COFFEE, Elem(2, 2): TEMP_STEAM}
+TEMP_REPORT_MAP = {
+    #Elem(0, 2): TEMP_COFFEE,
+    #Elem(2, 2): TEMP_STEAM
+}
 
 # R
 # 40 00 00 20: Message
@@ -181,9 +190,13 @@ STATUS_MAP = {
     Elem(17): KEY_ACTIVE,
     Elem(19, 2): CURRENT_PULSE_COUNT,
     # Elem(23): POWER,
+    Elem(24): WATER_RESERVOIR_CONTACT,
     Elem(27): HEATING_STATE,
-    # Elem(28, 2): TEMP_COFFEE,
-    # Elem(30, 2): TEMP_STEAM,
+    Elem(28, 2): TEMP_COFFEE,
+    Elem(30, 2): TEMP_STEAM,
+    Elem(32): MYSTERY_1,
+    Elem(33): MYSTERY_2,
+    Elem(34): STEAM_BOILER_ENABLE,
 }
 
 HEATING_VALUES = {
@@ -222,20 +235,21 @@ CONFIG_MAP = {
     Elem(0): POWER,
     Elem(7, 2): TSET_COFFEE,
     Elem(9, 2): TSET_STEAM,
-    Elem(11): ENABLE_PREBREWING,
-    Elem(12): PREBREWING_TON_K1,
-    Elem(13): PREBREWING_TON_K2,
-    Elem(14): PREBREWING_TON_K3,
-    Elem(15): PREBREWING_TON_K4,
-    Elem(16): PREBREWING_TOFF_K1,
-    Elem(17): PREBREWING_TOFF_K2,
-    Elem(18): PREBREWING_TOFF_K3,
-    Elem(19): PREBREWING_TOFF_K4,
-    Elem(20, 2): DOSE_K1,
-    Elem(22, 2): DOSE_K2,
-    Elem(24, 2): DOSE_K3,
-    Elem(26, 2): DOSE_K4,
-    Elem(28, 2): DOSE_K5,
+    Elem(11): PREBREW_FLAG,
+    Elem(CALCULATED_VALUE): ENABLE_PREBREWING,
+    Elem(12): (PREBREWING, TON, "k1"),
+    Elem(13): (PREBREWING, TON, "k2"),
+    Elem(14): (PREBREWING, TON, "k3"),
+    Elem(15): (PREBREWING, TON, "k4"),
+    Elem(16): (PREBREWING, TOFF, "k1"),
+    Elem(17): (PREBREWING, TOFF, "k2"),
+    Elem(18): (PREBREWING, TOFF, "k3"),
+    Elem(19): (PREBREWING, TOFF, "k4"),
+    Elem(20, 2): (DOSE, "k1"),
+    Elem(22, 2): (DOSE, "k2"),
+    Elem(24, 2): (DOSE, "k3"),
+    Elem(26, 2): (DOSE, "k4"),
+    Elem(28, 2): (DOSE, "k5"),
     Elem(30): DOSE_HOT_WATER,
 }
 
@@ -263,42 +277,58 @@ CONFIG_MAP = {
 
 AUTO_SCHED_MAP = {
     Elem(0): AUTO_BITFIELD,
-    Elem(1): SUN_ON,
-    Elem(2): SUN_OFF,
-    Elem(3): MON_ON,
-    Elem(4): MON_OFF,
-    Elem(5): TUE_ON,
-    Elem(6): TUE_OFF,
-    Elem(7): WED_ON,
-    Elem(8): WED_OFF,
-    Elem(9): THU_ON,
-    Elem(10): THU_OFF,
-    Elem(11): FRI_ON,
-    Elem(12): FRI_OFF,
-    Elem(13): SAT_ON,
-    Elem(14): SAT_OFF,
+    Elem(1): (MON, ON, HOUR),
+    Elem(2): (MON, OFF, HOUR),
+    Elem(3): (TUE, ON, HOUR),
+    Elem(4): (TUE, OFF, HOUR),
+    Elem(5): (WED, ON, HOUR),
+    Elem(6): (WED, OFF, HOUR),
+    Elem(7): (THU, ON, HOUR),
+    Elem(8): (THU, OFF, HOUR),
+    Elem(9): (FRI, ON, HOUR),
+    Elem(10): (FRI, OFF, HOUR),
+    Elem(11): (SAT, ON, HOUR),
+    Elem(12): (SAT, OFF, HOUR),
+    Elem(13): (SUN, ON, HOUR),
+    Elem(14): (SUN, OFF, HOUR),
+    Elem(15): (MON, ON, MIN),
+    Elem(16): (MON, OFF, MIN),
+    Elem(17): (TUE, ON, MIN),
+    Elem(18): (TUE, OFF, MIN),
+    Elem(19): (WED, ON, MIN),
+    Elem(20): (WED, OFF, MIN),
+    Elem(21): (THU, ON, MIN),
+    Elem(22): (THU, OFF, MIN),
+    Elem(23): (FRI, ON, MIN),
+    Elem(24): (FRI, OFF, MIN),
+    Elem(25): (SAT, ON, MIN),
+    Elem(26): (SAT, OFF, MIN),
+    Elem(27): (SUN, ON, MIN),
+    Elem(28): (SUN, OFF, MIN),
+    Elem(CALCULATED_VALUE): MON,
+    Elem(CALCULATED_VALUE): TUE,
+    Elem(CALCULATED_VALUE): WED,
+    Elem(CALCULATED_VALUE): THU,
+    Elem(CALCULATED_VALUE): FRI,
+    Elem(CALCULATED_VALUE): SAT,
+    Elem(CALCULATED_VALUE): SUN,
+}
+
+AUTO_ENABLE_MAP = {
+    Elem(0): AUTO_BITFIELD,
 }
 
 AUTO_BITFIELD_MAP = {
-    0: GLOBAL_AUTO,
-    1: MON_AUTO,
-    2: TUE_AUTO,
-    3: WED_AUTO,
-    4: THU_AUTO,
-    5: FRI_AUTO,
-    6: SAT_AUTO,
-    7: SUN_AUTO,
+    0: (GLOBAL, AUTO),
+    1: (MON, AUTO),
+    2: (TUE, AUTO),
+    3: (WED, AUTO),
+    4: (THU, AUTO),
+    5: (FRI, AUTO),
+    6: (SAT, AUTO),
+    7: (SUN, AUTO),
 }
 
-# W: Write
-# 00 00 00 01 : Message
-# OK: Result
-# 72: Check byte
-
-# WRITE_RESULT_MAP = {
-#     Elem(0, 1, type=Elem.STRING): WRITE_RESULT,
-# }
-
 # App: R 01 00 00 11 D5
 # Machine:
 # R
@@ -347,10 +377,10 @@ DATETIME_MAP = {
 # 00000032: Completed Hot Water
 
 DRINK_STATS_MAP = {
-    Elem(0, 4): DRINKS_K1,
-    Elem(4, 4): DRINKS_K2,
-    Elem(8, 4): DRINKS_K3,
-    Elem(12, 4): DRINKS_K4,
+    Elem(0, 4): (DRINKS, "k1"),
+    Elem(4, 4): (DRINKS, "k2"),
+    Elem(8, 4): (DRINKS, "k3"),
+    Elem(12, 4): (DRINKS, "k4"),
     Elem(16, 4): CONTINUOUS,
     Elem(20, 4): TOTAL_COFFEE,
     Elem(24, 4): HOT_WATER,
@@ -363,19 +393,19 @@ DRINK_STATS_MAP = {
 
 GATEWAY_DRINK_MAP = {
     -1: FLUSHING_OFFSET,
-    0: DRINKS_K1_OFFSET,
-    1: DRINKS_K2_OFFSET,
-    2: DRINKS_K3_OFFSET,
-    3: DRINKS_K4_OFFSET,
+    0: (DRINKS, "k1", OFFSET),
+    1: (DRINKS, "k2", OFFSET),
+    2: (DRINKS, "k3", OFFSET),
+    3: (DRINKS, "k4", OFFSET),
     4: CONTINUOUS_OFFSET,
 }
 
 DRINK_OFFSET_MAP = {
     TOTAL_FLUSHING: FLUSHING_OFFSET,
-    DRINKS_K1: DRINKS_K1_OFFSET,
-    DRINKS_K2: DRINKS_K2_OFFSET,
-    DRINKS_K3: DRINKS_K3_OFFSET,
-    DRINKS_K4: DRINKS_K4_OFFSET,
+    (DRINKS, "k1"): (DRINKS, "k1", OFFSET),
+    (DRINKS, "k2"): (DRINKS, "k2", OFFSET),
+    (DRINKS, "k3"): (DRINKS, "k3", OFFSET),
+    (DRINKS, "k4"): (DRINKS, "k4", OFFSET),
     CONTINUOUS: CONTINUOUS_OFFSET,
 }
 
@@ -417,37 +447,136 @@ USAGE_MAP = {
     Elem(20, 4): WATER_ON_SECONDS,
 }
 
+# 3230322E39DF46: 202.9°F
+# 20: Space
+# DBDB: Boiler states
+# 30393A32386100: 09:28a
+# 42555A5A2020202020202020: BUZZ
+# 543A3037: T:07
+
+
+FRONT_DISPLAY_MAP = {
+    Elem(0, 16, Elem.STRING): FRONT_PANEL_DISPLAY,
+}
+
+# 03
+# E7
+# 02
+# D8
+# 04
+# F8
+# 09
+# 5B
+# 02
+# C2
+# 03
+# 04
+# 08
+# 9E
+# 02
+# 49
+# 00
+# F5
+
+MYSTERY_MAP = {
+    Elem(0): (VAL, 1),
+    Elem(1): (VAL, 2),
+    Elem(2): (VAL, 3),
+    Elem(3): (VAL, 4),
+    Elem(4): (VAL, 5),
+    Elem(5): (VAL, 6),
+    Elem(6): (VAL, 7),
+    Elem(7): (VAL, 8),
+    Elem(8): (VAL, 9),
+    Elem(9): (VAL, 10),
+    Elem(10): (VAL, 11),
+    Elem(11): (VAL, 12),
+    Elem(12): (VAL, 13),
+    Elem(13): (VAL, 14),
+    Elem(14): (VAL, 15),
+    Elem(15): (VAL, 16),
+    Elem(16): (VAL, 17),
+    Elem(17): (VAL, 18),
+}
+
+PREINFUSION_MAP = {
+    Elem(CALCULATED_VALUE): ENABLE_PREINFUSION,
+    Elem(0): (PREINFUSION, "k1"),
+    Elem(1): (PREINFUSION, "k2"),
+    Elem(2): (PREINFUSION, "k3"),
+    Elem(3): (PREINFUSION, "k4"),
+}
+
+# 64
+# 00
+# 64
+# 00
+# 70
+# 00
+# 00
+# 00
+# 00
+# 01
+# 01
+# 69: Group Offset (0-200 C, 0-360 F, +/- from the midpoint)
+# 00
+# 55
+# 00
+# 00
+
+FACTORY_CONFIG_MAP = {
+    Elem(4): PID_OFFSET,
+    Elem(6): WATER_FILTER_LITERS,
+    Elem(11, 2): BREW_GROUP_OFFSET,
+    Elem(13): T_UNIT,
+}
 
 class Msg:
-    GET_STATUS = 0
-    GET_CONFIG = 1
-    GET_AUTO_SCHED = 2
-    SET_POWER = 3
-    GET_TEMP_REPORT = 4
-    SET_AUTO_ON = 4
-    SET_AUTO_OFF = 5
-    GET_SER_NUM = 6
-    GET_DATETIME = 7
-    SET_COFFEE_TEMP = 8
-    SET_STEAM_TEMP = 9
-    SET_PREBREWING_ENABLE = 10
-    SET_AUTO_SCHED = 11
-    SET_DOSE = 12
-    SET_DOSE_HOT_WATER = 13
-    SET_PREBREW_TIMES = 14
-    GET_DRINK_STATS = 15
-    GET_WATER_FLOW = 16
-    GET_USAGE_STATS = 17
+    GET_STATUS = "get_status"
+    GET_SHORT_STATUS = "get_short_status"
+    GET_CONFIG = "get_config"
+    GET_AUTO_ON_OFF_TIMES = "get_auto_sched"
+    SET_AUTO_ON_OFF_TIMES = "set_auto_on_off_times"
+    SET_POWER = "set_power"
+    GET_TEMP_REPORT = "get_temp_report"
+    GET_AUTO_ON_OFF_ENABLE = "get_auto_on_off_enable"
+    SET_AUTO_ON_OFF_ENABLE = "set_auto_on_off_enable"
+    GET_SER_NUM = "get_ser_num"
+    GET_DATETIME = "get_datetime"
+    SET_COFFEE_TEMP = "set_coffee_temp"
+    SET_STEAM_TEMP = "set_steam_temp"
+    SET_PREBREWING_ENABLE = "set_prebrewing_enable"
+    SET_DOSE = "set_dose"
+    SET_DOSE_HOT_WATER = "set_dose_hot_water"
+    SET_PREBREW_TIMES = "set_prebrew_times"
+    GET_DRINK_STATS = "get_drink_stats"
+    GET_WATER_FLOW = "get_water_flow"
+    GET_USAGE_STATS = "get_usage_stats"
+    GET_FRONT_DISPLAY = "get_front_display"
+    GET_MYSTERY = "get_mystery"
+    SET_START_BACKFLUSH = "set_start_backflush"
+    GET_STATUS_MYSTERY = "get_status_mystery"
+    GET_PREINFUSION_TIMES = "get_preinfusion_times"
+    SET_PREINFUSION_TIME = "set_preinfusion_time"
+    SET_STEAM_BOILER_ENABLE = "set_steam_boiler_enable"
+    GET_FACTORY_CONFIG = "get_factory_config"
 
-    """Dose command starts at 0x14 for the first key and increments by 2 thereafter."""
+    """Auto on/off address starts at 0x11 for Monday and increments by 4 thereafter."""
+    AUTO_ON_OFF_HOUR_BASE = 0x11
+    AUTO_ON_OFF_MIN_BASE = 0x20
+
+    """Dose address starts at 0x14 for the first key and increments by 2 thereafter."""
     DOSE_KEY_BASE = 0x14
 
-    """Prebrew on base."""
+    """Prebrew on address base."""
     PREBREW_ON_BASE = 0x0C
 
-    """Prebrew off base."""
+    """Prebrew off address base."""
     PREBREW_OFF_BASE = 0x10
 
+    """Preinfusion address base."""
+    PREINFUSION_BASE = 0xE2
+
     READ = "R"
     WRITE = "W"
     STREAM = "Z"
@@ -478,25 +607,36 @@ class Msg:
 
 MSGS = {
     # Reads
-    Msg.GET_STATUS: Msg(Msg.READ, "40000020", STATUS_MAP),
+    Msg.GET_STATUS: Msg(Msg.READ, "40000023", STATUS_MAP),
+    Msg.GET_SHORT_STATUS: Msg(Msg.READ, "40000020", None),
     Msg.GET_CONFIG: Msg(Msg.READ, "0000001F", CONFIG_MAP),
-    Msg.GET_AUTO_SCHED: Msg(Msg.READ, "0310001D", AUTO_SCHED_MAP),
+    Msg.GET_AUTO_ON_OFF_TIMES: Msg(Msg.READ, "0310001D", AUTO_SCHED_MAP),
+    Msg.GET_AUTO_ON_OFF_ENABLE: Msg(Msg.READ, "03100001", AUTO_ENABLE_MAP),
     Msg.GET_TEMP_REPORT: Msg(Msg.READ, "401C0004", TEMP_REPORT_MAP),
     Msg.GET_SER_NUM: Msg(Msg.READ, "01000011", SER_NUM_MAP),
     Msg.GET_DATETIME: Msg(Msg.READ, "03000007", DATETIME_MAP),
     Msg.GET_DRINK_STATS: Msg(Msg.READ, "0020002C", DRINK_STATS_MAP),
     Msg.GET_WATER_FLOW: Msg(Msg.STREAM, "60000016", WATER_FLOW_MAP),
     Msg.GET_USAGE_STATS: Msg(Msg.READ, "00500018", USAGE_MAP),
+    Msg.GET_FRONT_DISPLAY: Msg(Msg.READ, "60EF0010", FRONT_DISPLAY_MAP),
+    Msg.GET_MYSTERY: Msg(Msg.READ, "60DA0012", MYSTERY_MAP),
+    Msg.GET_STATUS_MYSTERY: Msg(Msg.READ, "40220001", None),
+    Msg.GET_PREINFUSION_TIMES: Msg(Msg.READ, "00E20004", PREINFUSION_MAP),
+    Msg.GET_FACTORY_CONFIG: Msg(Msg.READ, "4070000E", FACTORY_CONFIG_MAP),
     # Writes
     Msg.SET_POWER: Msg(Msg.WRITE, "00000001", None),
     Msg.SET_COFFEE_TEMP: Msg(Msg.WRITE, "00070002", None),
     Msg.SET_STEAM_TEMP: Msg(Msg.WRITE, "00090002", None),
     Msg.SET_PREBREWING_ENABLE: Msg(Msg.WRITE, "000B0001", None),
-    Msg.SET_AUTO_SCHED: Msg(Msg.WRITE, "0310001D", None),
+    Msg.SET_AUTO_ON_OFF_TIMES: Msg(Msg.WRITE, "03100002", None),
+    Msg.SET_AUTO_ON_OFF_ENABLE: Msg(Msg.WRITE, "03100001", None),
     # Write config for keys (second byte is base + the key number and will be replaced)
     Msg.SET_DOSE: Msg(Msg.WRITE, "00140002", None),
     Msg.SET_DOSE_HOT_WATER: Msg(Msg.WRITE, "001E0001", None),
     Msg.SET_PREBREW_TIMES: Msg(Msg.WRITE, "000C0001", None),
+    Msg.SET_START_BACKFLUSH: Msg(Msg.WRITE, "00E10001", None),
+    Msg.SET_PREINFUSION_TIME: Msg(Msg.WRITE, "00E20001", None),
+    Msg.SET_STEAM_BOILER_ENABLE: Msg(Msg.WRITE, "00E10001", None),
 }
 
 # 0000:001f - Config
